@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import abc
 from enum import Enum, EnumMeta
+import importlib.util
 from inspect import isclass
 import io
 import itertools
@@ -16,6 +17,7 @@ import numpy as np
 import pinocchio as pin
 
 from timor.utilities import logging
+from timor.utilities.jsonable import JSONable_mixin
 from timor.utilities.spatial import rotX
 from timor.utilities.transformation import Transformation, TransformationLike
 
@@ -85,7 +87,7 @@ def _geometry_type2class(geometry_type: GeometryType) -> Type[Geometry]:
         return mapping[GeometryType[geometry_type]]
 
 
-class Geometry(abc.ABC):
+class Geometry(abc.ABC, JSONable_mixin):
     """A class describing geometries.
 
     A geometry is defined by a type (see GeometryType), an expansion (defined by custom parameters) and a
@@ -123,33 +125,33 @@ class Geometry(abc.ABC):
         return json.dumps(self.serialized, indent=2)
 
     @classmethod
-    def from_json_data(cls, description: Union[List, Dict[str, any]],
-                       package_dir: Optional[Path] = None) -> Geometry:
+    def from_json_data(cls, d: Union[List, Dict[str, any]], package_dir: Optional[Path] = None,
+                       *args, **kwargs) -> Geometry:
         """
         Takes a serialized geometry specification and returns the according Geometry instance.
 
-        :param description: A serialized geometry
+        :param d: A serialized geometry
         :param package_dir: The top directory to which the mesh file paths are relative to. Necessary for meshes (only).
         """
-        if isinstance(description, (list, tuple, set)):
-            if len(description) == 1:
-                description = description[0]
+        if isinstance(d, (list, tuple, set)):
+            if len(d) == 1:
+                d = d[0]
             else:
-                return ComposedGeometry((cls.from_json_data(d, package_dir) for d in description))
+                return ComposedGeometry((cls.from_json_data(d, package_dir) for d in d))
 
         if cls is not Geometry:
             raise AttributeError("Class {} does not support this method. Call it on the Geometry class.".format(cls))
-        desc = description.copy()
+        desc = d.copy()
         class_ref = _geometry_type2class(desc.pop('type'))
 
         if class_ref is Mesh:
-            if package_dir is None:
-                raise ValueError("If your geometry contains a mesh, you need to provide the package_dir")
-            desc['parameters']['package_dir'] = package_dir
-            # The "package://" prefix might be given, but is not mandatory.
-            desc['parameters']['file'] = re.sub('package://', '', desc['parameters']['file'])
-            if not (package_dir / Path(desc['parameters']['file'])).exists():
-                raise ValueError("Mesh file {} does not exist".format(desc['parameters']['file']))
+            if package_dir is not None:
+                # The "package://" prefix might be given, but is not mandatory.
+                desc['parameters']['file'] = re.sub('package://', '', desc['parameters']['file'])
+                desc['parameters']['file'] = package_dir / Path(desc['parameters']['file'])
+
+            if not (Path(desc['parameters']['file'])).exists():
+                raise FileNotFoundError("Mesh file {} does not exist".format(desc['parameters']['file']))
 
         return class_ref(**desc)
 
@@ -189,8 +191,10 @@ class Geometry(abc.ABC):
                                                                               geo.placement.translation))
         if isinstance(intermediate, Mesh):
             logging.debug("Restoring mesh file information from pinocchio geometry object")
-            intermediate.parameters['file'] = geo.meshPath
-            intermediate.parameters['scale'] = geo.meshScale
+            params = intermediate.parameters
+            params['file'] = geo.meshPath
+            params['scale'] = geo.meshScale
+            intermediate.parameters = params  # Trigger setter
         return intermediate
 
     @property
@@ -210,7 +214,7 @@ class Geometry(abc.ABC):
         return [{
             'type': self.type.value,
             'parameters': self.parameters,  # This is a dict itself
-            'pose': self.placement.serialized
+            'pose': self.placement.to_json_data()
         }]
 
     @property
@@ -246,6 +250,19 @@ class Geometry(abc.ABC):
         """Must be implemented for Geometries, as hppfcl does not natively support it."""
         return self.__class__(self.parameters, self.placement)
 
+    def __eq__(self, other: Geometry):
+        """Compares two Geometry instances based on their type, parameters and placement."""
+        if isinstance(other, Geometry) and not isinstance(other, Mesh):
+            this_parameters = tuple(sorted(((k, v) for k, v in self.parameters.items()), key=lambda x: x[0]))
+            other_parameters = tuple(sorted(((k, v) for k, v in other.parameters.items()), key=lambda x: x[0]))
+            return self.type == other.type and this_parameters == other_parameters and self.placement == other.placement
+        else:
+            return NotImplemented
+
+    def __hash__(self):
+        """Hashes the Geometry instance based on its type, parameters and placement."""
+        return hash((self.type, tuple(hash(p) for p in self.parameters.items()), self.placement.to_json_string()))
+
 
 class Box(Geometry):
     """A simple geometry with a box extending in x, y and z direction."""
@@ -333,8 +350,14 @@ class Cylinder(Geometry):
     @parameters.setter
     def parameters(self, parameters: Dict[str, float]):
         """Unpacks to r, z"""
-        self._r = parameters['r']
-        self._z = parameters['z']
+        try:
+            self._r = parameters['r']
+        except KeyError:
+            self._r = parameters['radius']
+        try:
+            self._z = parameters['z']
+        except KeyError:
+            self._z = parameters['length']
 
     @property
     def urdf_properties(self) -> Tuple[str, Dict[str, float]]:
@@ -415,13 +438,13 @@ class Mesh(Geometry):
     type: GeometryType = GeometryType.MESH
     _filepath: Path
     _scale: Union[float, np.ndarray]
-    package_dir: Optional[Path]
+    _colors: Optional[np.ndarray] = None
 
     @classmethod
     def from_hppfcl(cls, fcl: hppfcl.CollisionObject) -> Geometry:
         """Wraps a hppfcl Mesh in a Mesh instance."""
         logging.debug("Creating mesh from hppfcl, will lose possible file location information")
-        return cls({'file': '', 'package_dir': ''}, fcl.getTransform(), fcl.collisionGeometry())
+        return cls({'file': ''}, fcl.getTransform(), fcl.collisionGeometry())
 
     @staticmethod
     def read_wrl(file: Path) -> Tuple[Tuple[np.ndarray, ...], Tuple[np.ndarray, ...]]:
@@ -468,15 +491,20 @@ class Mesh(Geometry):
             return geom
         else:
             mesh_loader = hppfcl.MeshLoader()
+            if self.abs_filepath.suffix == ".ply":
+                if importlib.util.find_spec("trimesh"):
+                    import trimesh
+                    m = trimesh.load(self.abs_filepath)
+                    # Map uint8 to 0. .. 1. float value; cut of alpha
+                    self._colors = m.visual.vertex_colors[:, :3] / 255
+                else:
+                    logging.info("Install trimesh / timor full to enable color meshes.")
             return mesh_loader.load(str(self.abs_filepath), self.scale)
 
     @property
     def abs_filepath(self) -> Path:
-        """The absolute filepath is the combination of package path and relative path of the mesh file."""
-        if self.package_dir is None:
-            raise AttributeError("package_dir not set")
-
-        return self.package_dir / self.filepath
+        """The absolute filepath of the mesh file."""
+        return self.filepath
 
     @property
     def parameters(self) -> Dict[str, Union[str, float]]:
@@ -489,20 +517,27 @@ class Mesh(Geometry):
 
     @parameters.setter
     def parameters(self, parameters: Dict[str, any]):
-        """Unpacks to filepath and scale - also expects a package_dir to be able to load the mesh."""
-        self.package_dir = parameters['package_dir']
+        """Unpacks to filepath and scale."""
         self._filepath = Path(parameters['file'])
+        if 'package_dir' in parameters:
+            self._filepath = Path(parameters['package_dir']) / self._filepath
         self._scale = np.asarray(parameters.get('scale', 1.0))
 
     @property
     def urdf_properties(self) -> Tuple[str, Dict[str, Union[str, List[float]]]]:
         """Can only be saved as a dictionary if the mesh was loaded from a file and the file location is still known."""
-        return 'mesh', {'filename': 'package://' + str(self.filepath), 'scale': self.scale.tolist()}
+        return 'mesh', {'filename': str(self.filepath), 'scale': self.scale.tolist()}
 
     @property
     def viz_object(self) -> Tuple[meshcat.geometry.Geometry, Transformation]:
         """Returns a meshcat mesh"""
-        return pin.visualize.meshcat_visualizer.loadMesh(self.collision_geometry), self.placement
+        geom = pin.visualize.meshcat_visualizer.loadMesh(self.collision_geometry), self.placement
+        if self._colors is not None:
+            if geom[0].vertices.shape == self._colors.shape:
+                geom[0].color = self._colors
+            else:
+                logging.warning("Color shape seems to not fit vertices.")
+        return geom
 
     @property
     def volume(self) -> float:
@@ -526,6 +561,24 @@ class Mesh(Geometry):
             return np.array([self._scale, self._scale, self._scale])
         return self._scale
 
+    def __deepcopy__(self, memodict={}):
+        """Must be implemented for Geometries, as hppfcl does not natively support it."""
+        params = self.parameters
+        return self.__class__(params, self.placement)
+
+    def __eq__(self, other: Geometry):
+        """We don't care about Mesh file paths, but equality of their collision geometries."""
+        if isinstance(other, Mesh):
+            return self.type == other.type \
+                and np.all(self.scale == other.scale) \
+                and self.placement == other.placement \
+                and self.collision_geometry == other.collision_geometry
+        return NotImplemented
+
+    def __hash__(self):
+        """Scale can't always be hashed as it can be a numpy array"""
+        return hash((self.type, np.array(self.scale).tobytes(), self.placement.to_json_string()))
+
 
 class ComposedGeometry(Geometry):
     """A composed geometry is a collection of geometries that are combined in one instance."""
@@ -575,7 +628,7 @@ class ComposedGeometry(Geometry):
         return [{
             'type': g.type.value,
             'parameters': g.parameters,
-            'pose': g.placement.serialized
+            'pose': g.placement.to_json_data()
         } for g in self.composing_geometries]
 
     @property
@@ -598,6 +651,17 @@ class ComposedGeometry(Geometry):
         """The volume of a composed geometry is the sum of the volumes of the composing geometries."""
         return sum(g.volume for g in self.composing_geometries)
 
+    def __eq__(self, other):
+        """Equality is defined as having the same composing geometries."""
+        if not isinstance(other, ComposedGeometry):
+            return False
+        return set(g for g in self.composing_geometries if not isinstance(g, EmptyGeometry)) == \
+            set(g for g in other.composing_geometries if not isinstance(g, EmptyGeometry))
+
+    def __hash__(self):
+        """Hash is defined as the hash of the composing geometries."""
+        return sum(hash(g) for g in self.composing_geometries if not isinstance(g, EmptyGeometry))
+
 
 class EmptyGeometry(Geometry):
     """This is a dummy geometry to being able to prevent setting a geometry to None."""
@@ -610,6 +674,8 @@ class EmptyGeometry(Geometry):
                  hppfcl_representation: Optional[hppfcl.CollisionGeometry] = None
                  ):
         """For an empty geometry, the parameters are not used."""
+        if parameters is not None:
+            logging.warning("EmptyGeometry does not use parameters, but they were given.")
         parameters = dict()
         super().__init__(parameters, pose, hppfcl_representation)
 
@@ -656,3 +722,13 @@ class EmptyGeometry(Geometry):
     def _make_collision_geometry(self) -> hppfcl.CollisionGeometry:
         """No Geometry, no collision object."""
         pass
+
+    def __eq__(self, other):
+        """Empty geometries are equal to other empty geometries."""
+        if isinstance(other, EmptyGeometry):
+            return True
+        return NotImplemented
+
+    def __hash__(self):
+        """Empty geometries are equal to other empty geometries."""
+        return hash(self.__class__.__name__)
