@@ -1,3 +1,4 @@
+import logging
 import time
 import unittest
 
@@ -19,8 +20,8 @@ class TestModuleAssembly(unittest.TestCase):
 
     def setUp(self) -> None:
         """Initialize some modules"""
-        self.json_file, self.package_dir = get_module_db_files('IMPROV')
-        self.db = ModulesDB.from_file(self.json_file, package_dir=self.package_dir)
+        self.json_file = get_module_db_files('IMPROV')
+        self.db = ModulesDB.from_json_file(self.json_file)
 
     def test_graph_features(self):
         """An assembly of modules is (given some abstraction) just a graph. Check some basic graph attributes:"""
@@ -73,6 +74,16 @@ class TestModuleAssembly(unittest.TestCase):
         self.assertIsNotNone(assembly.base_connector)
         self.assertEqual(len(assembly.module_instances), 2)
 
+        # Check that the assembly can be (de)serialized and we have a functional (in)equality
+        new_assembly = assembly.from_json_string(assembly.to_json_string())
+        self.assertEqual(assembly, new_assembly)
+        try:
+            new_assembly.add_random_from_db()
+        except LookupError:
+            logging.debug("No modules left to add. Removing one module for inequality test")
+            new_assembly.connections.pop()
+        self.assertNotEqual(assembly, new_assembly)
+
     def test_parameterizable_assembly(self):
         """Tests assemblies containing at least on parameterizable module."""
         base_header = ModuleHeader('B', 'Base')
