@@ -5,6 +5,7 @@ import numpy as np
 import numpy.testing as np_test
 import pinocchio as pin
 
+from timor.Module import ModuleAssembly, ModulesDB
 from timor.Robot import PinRobot, RobotBase
 from timor.task import Tolerance
 from timor.utilities import logging, prebuilt_robots, spatial
@@ -49,6 +50,16 @@ class PinocchioRobotSetup(unittest.TestCase):
             robot_one.set_base_placement(original)  # Should change nothing
             np_test.assert_array_almost_equal(original, robot_one.placement.homogeneous)
 
+        # Test displacement for a robot built from a module
+        db = ModulesDB.from_name('geometric_primitive_modules')
+        mini_assembly = ModuleAssembly(db, ['J2'], base_connector=(0, 'J2_proximal'))
+        jnt_displacement = (Transformation(spatial.rotX(np.pi)) @
+                            db.all_connectors_by_id[('J2', 'J2_proximal', 'J2_proximal')].connector2body @
+                            list(db.by_id['J2'].joints)[0].parent2joint).homogeneous
+        mini_robot = mini_assembly.to_pin_robot(base_placement=new_placement)
+        np_test.assert_array_almost_equal(mini_robot.model.jointPlacements[1].homogeneous,
+                                          new_placement.homogeneous @ jnt_displacement)
+
     def test_empty_robot(self):
         wrapper = pin.RobotWrapper(pin.Model())
         robot = PinRobot(wrapper=wrapper)
@@ -177,6 +188,10 @@ class PinocchioRobotSetup(unittest.TestCase):
         panda_tcp = pin.SE3(spatial.homogeneous(np.array([0, 0, .21]), spatial.rotZ(-np.pi / 4)[:3, :3]))
         robot.add_tcp_frame('panda_joint7', panda_tcp)
 
+        def translation_cost(_r: RobotBase, _q: np.ndarray, _g: np.ndarray):
+            """A custom cost function to optimize for translation only"""
+            return _r.fk(_q, 'tcp').distance(_g).translation_euclidean
+
         error_count = {"scipy": 0, "jacobian": 0}
         N = 100
         n = 0
@@ -195,7 +210,7 @@ class PinocchioRobotSetup(unittest.TestCase):
             n += 1  # Keep track of how many examples were really worked with
             robot.model.effortLimit = inf_limits  # Make it impossible to fail due to the torque limits
             robot.update_configuration(pin.neutral(robot.model))
-            conf, success = robot.ik_scipy(ToleratedPose(goal, scipy_tolerance), check_static_torques=True)
+            conf, success = robot.ik_scipy(ToleratedPose(goal, scipy_tolerance), ik_cost_function=translation_cost)
             if success:
                 # ik_scipy optimizes only for position, therefore the Cartesian XYZ tolerance
                 self.assertTrue(scipy_tolerance.valid(goal, robot.fk(conf)))
@@ -205,7 +220,7 @@ class PinocchioRobotSetup(unittest.TestCase):
                 error_count['scipy'] += 1
 
             robot.update_configuration(pin.neutral(robot.model))  # Prevent using the scipy result as initial guess
-            conf, success = robot.ik_jacobian(ToleratedPose(goal, jacobian_tolerance), check_static_torques=True)
+            conf, success = robot.ik_jacobian(ToleratedPose(goal, jacobian_tolerance))
             if success:
                 np_test.assert_array_almost_equal(goal.homogeneous, robot.fk(conf).homogeneous, decimal=2)
                 self.assertTrue(jacobian_tolerance.valid(goal, robot.fk(conf)))
@@ -272,6 +287,13 @@ class PinocchioRobotSetup(unittest.TestCase):
         self.assertIsNotNone(robot.visualize())
         self.assertIsNotNone(robot.visualize_self_collisions())
 
+    def test_robot_manip(self):
+        robot = PinRobot.from_urdf(self.urdf, self.package_dir)
+        # Some best guess singular / non-singular panda poses
+        self.assertAlmostEqual(0., robot.manipulability_index(np.asarray((0., 0., 0., 0., 0., 0., 0.))))
+        self.assertGreater(
+            0.5, robot.manipulability_index(np.asarray((0., -np.pi / 4, 0., np.pi / 2, 0., np.pi / 2, 0.))))
+
 
 if __name__ == '__main__':
     unittest.main()
