@@ -1,4 +1,6 @@
+from copy import deepcopy
 import json
+import os
 from pathlib import Path
 import pickle
 import random
@@ -12,10 +14,11 @@ import pinocchio as pin
 
 from timor import Geometry, Transformation
 from timor import Robot
+from timor.Geometry import Mesh
 from timor.Module import ModuleAssembly, ModulesDB
 from timor.task import Constraints, CostFunctions, Obstacle, Solution, Task, Tolerance
 from timor.utilities import file_locations, prebuilt_robots, logging
-from timor.utilities.file_locations import get_module_db_files, schema_dir
+from timor.utilities.file_locations import schema_dir
 from timor.utilities.prebuilt_robots import random_assembly
 from timor.utilities.schema import get_schema_validator
 from timor.utilities.tolerated_pose import ToleratedPose
@@ -212,6 +215,55 @@ class SerializationTests(unittest.TestCase):
         self.constraints = [goal_order, joint_constraints, base_placement, collision, eef_constraint_pose,
                             eef_constraint_v, eef_constraint_all]
 
+    def test_modules_db_serialized(self):
+        """Tests if all modules dbs can be serialized and deserialized"""
+        for db_name in ('IMPROV', 'PROMODULAR', 'modrob-gen2'):
+            logging.debug(f"Testing with {db_name}")
+            db = ModulesDB.from_name(db_name)
+            self.assertEqual(db.name, db_name)  # Ensure name correctly inferred from path
+            for m in db:
+                pickle_mod = pickle.loads(pickle.dumps(m))
+                deepcopy_mod = deepcopy(m)
+                self.assertEqual(m, pickle_mod)
+                self.assertEqual(m, deepcopy_mod)
+            db_json = db.to_json_data()
+            new_db = ModulesDB.from_json_data(db_json)
+            new_db._name = db.name
+            self.assertEqual(db, new_db)
+            deep_copy_db = deepcopy(db)
+            self.assertEqual(db, deep_copy_db)
+            s = pickle.dumps(db)
+            pickle_db = pickle.loads(s)
+            self.assertEqual(db, pickle_db)
+
+            # Test different to_json_file modes
+            with tempfile.TemporaryDirectory() as tmp_dir:
+                os.makedirs(Path(tmp_dir, db.name), exist_ok=True)
+                db_file = Path(tmp_dir, f"{db.name}/modules.json")
+                # Broken - error as not existent file
+                with self.assertRaises(FileNotFoundError):
+                    db.to_json_file(db_file, handle_missing_assets='error')
+                # Broken - no copy
+                db.to_json_file(db_file, handle_missing_assets="warning")
+                with self.assertRaises(FileNotFoundError):
+                    new_db = ModulesDB.from_json_file(db_file)
+                # Broken - no copy
+                db.to_json_file(db_file, handle_missing_assets='ignore')
+                with self.assertRaises(FileNotFoundError):
+                    new_db = ModulesDB.from_json_file(db_file)
+                # Should work with copy
+                db.to_json_file(db_file, handle_missing_assets='copy')
+                new_db = ModulesDB.from_json_file(db_file)
+                # self.assertTrue(equivalent_set(db, new_db, equivalent_module))  # Other filepath -> not same geometry
+
+            with tempfile.TemporaryDirectory() as tmp_dir:  # Cleanup copied files
+                os.makedirs(Path(tmp_dir, db.name), exist_ok=True)
+                db_file = Path(tmp_dir, f"{db.name}/modules.json")
+                # Should work with symlink
+                db.to_json_file(db_file, handle_missing_assets='symlink')
+                new_db = ModulesDB.from_json_file(db_file)
+                # self.assertTrue(equivalent_set(db, new_db, equivalent_module))  # Other filepath -> not same geometry
+
     def test_assembly_to_serialized(self):
         """Tests both, assembly to pickle and to json"""
         for db_name in ('IMPROV', 'PROMODULAR', 'modrob-gen2'):
@@ -252,7 +304,7 @@ class SerializationTests(unittest.TestCase):
                 pin_geometry_models_functionally_equal(robot, reconstructed_robot)
 
     def test_assembly_to_urdf(self):
-        _, db_assets = file_locations.get_module_db_files('IMPROV')
+        db_file = file_locations.get_module_db_files('IMPROV')
         db = ModulesDB.from_name('IMPROV')
         for _ in range(5):
             assembly = prebuilt_robots.random_assembly(n_joints=6, module_db=db)
@@ -260,10 +312,17 @@ class SerializationTests(unittest.TestCase):
 
             r1 = assembly.to_pin_robot()
             r1._remove_home_collisions()  # This is done per default when loading from URDF
-            with tempfile.NamedTemporaryFile() as tmp_urdf:
-                urdf = assembly.to_urdf('test_robot', Path(tmp_urdf.name), replace_wrl=True)
-                tmp_urdf.flush()
-                r2 = Robot.PinRobot.from_urdf(Path(tmp_urdf.name), db_assets)
+            with tempfile.TemporaryDirectory() as tmp_dir:
+                tmp_urdf = Path(tmp_dir, 'test.urdf')
+                urdf = assembly.to_urdf('test_robot', Path(tmp_urdf),
+                                        replace_wrl=True, handle_missing_assets="warning")
+                with self.assertRaises(ValueError):
+                    Robot.PinRobot.from_urdf(Path(tmp_urdf), tmp_urdf.parent)
+                with self.assertRaises(FileNotFoundError):
+                    assembly.to_urdf('test_robot', Path(tmp_urdf), replace_wrl=True, handle_missing_assets="error")
+                urdf = assembly.to_urdf('test_robot', Path(tmp_urdf),
+                                        replace_wrl=True, handle_missing_assets="symlink")
+                r2 = Robot.PinRobot.from_urdf(Path(tmp_urdf), tmp_urdf.parent)
                 self.assertTrue(pin_models_functionally_equal(r1.model, r2.model))
                 self.assertLess(pin_geometry_models_functionally_equal(r1, r2), 0.01,
                                 msg=f"Varying self-collisions for assembly {assembly}")
@@ -327,45 +386,64 @@ class SerializationTests(unittest.TestCase):
         self.assertIsNotNone(json.dumps(solution_dict))
         self.assertIsNotNone(pickle.dumps(solution_dict))
         _, validator = get_schema_validator(schema_dir.joinpath("SolutionSchema.json"))
+        deserialized_sol = Solution.SolutionBase.from_json_data(solution_dict, {task.id: task})
+        deepcopy_sol = deepcopy(solution)
+        pickle_sol = pickle.loads(pickle.dumps(solution))
+
         # make sure is storable
         with tempfile.NamedTemporaryFile(mode="w+") as t:
             json.dump(solution_dict, t)
             t.flush()
             with open(t.name) as f:
                 validator.validate(json.load(f))
-            loaded_sol = Solution.SolutionBase.from_json_file(Path(t.name), Path("/"), {task.id: task})
+            loaded_sol = Solution.SolutionBase.from_json_file(Path(t.name), {task.id: task})
 
         # Also check the custom task
-        with tempfile.NamedTemporaryFile(mode="w+") as t:
-            task.to_json_file(t.name)
-            t.flush()
+        with tempfile.TemporaryDirectory() as t:
+            task.to_json_file(Path(t))
+            with self.assertRaises(ValueError):
+                task.to_json_file(Path(t).joinpath(f"{task.id}_5.json"))
 
-        self.assertEqual(solution.valid, loaded_sol.valid)
-        # Depends on robot, trajectory, position, cost function -> most important parts of solution tested
-        self.assertEqual(solution.cost, loaded_sol.cost)
-        self.assertEqual(solution.trajectory, loaded_sol.trajectory)
-        self.assertEqual(solution.header, loaded_sol.header)
+        for sol in (deserialized_sol, deepcopy_sol, pickle_sol, loaded_sol):
+            self.assertEqual(solution.valid, sol.valid)
+            # Depends on robot, trajectory, position, cost function -> most important parts of solution tested
+            self.assertEqual(solution.cost, sol.cost)
+            self.assertEqual(solution.trajectory, sol.trajectory)
+            self.assertEqual(solution.header, sol.header)
 
         with tempfile.NamedTemporaryFile(mode="w+") as t:
             json.dump({"test": "json_validator"}, t)
             t.flush()
             with self.assertRaises(ValueError):
-                Solution.SolutionBase.from_json_file(Path(t.name), Path("/"), {task.id: task})
+                Solution.SolutionBase.from_json_file(Path(t.name), {task.id: task})
 
     def test_load_then_dump_task(self):
         for task_file in self.task_files:
-            task = Task.Task.from_json_file(task_file, self.assets)
+            task = Task.Task.from_json_file(task_file)
+            self.assertEqual(task, deepcopy(task))
+            self.assertEqual(task, pickle.loads(pickle.dumps(task)))
+            self.assertEqual(task, Task.Task.from_json_data(task.to_json_data()))
 
-            with tempfile.NamedTemporaryFile() as tmp_json_dump:
-                task.to_json_file(Path(tmp_json_dump.name))
-                tmp_json_dump.flush()  # Necessary to certainly write to tempfile!
-                another_copy = Task.Task.from_json_file(Path(tmp_json_dump.name), self.assets)
+            with tempfile.TemporaryDirectory() as tmp_dir:
+                tmp_json_file = Path(tmp_dir).joinpath(f"{task.id}.json")
+                if any(isinstance(o.visual, Mesh) for o in task.obstacles) or \
+                   any(isinstance(o.collision, Mesh) for o in task.obstacles):
+                    with self.assertRaises(FileNotFoundError):
+                        task.to_json_file(tmp_dir, handle_missing_assets="error")
+                    task.to_json_file(tmp_dir, handle_missing_assets="warning")
+                    self.assertTrue(tmp_json_file.exists())
+                    with self.assertRaises(FileNotFoundError):
+                        Task.Task.from_json_file(tmp_json_file)
+                    task.to_json_file(tmp_dir, handle_missing_assets="ignore")
+                    self.assertTrue(tmp_json_file.exists())
+                    with self.assertRaises(FileNotFoundError):
+                        Task.Task.from_json_file(tmp_json_file)
+
+                task.to_json_file(Path(tmp_dir), handle_missing_assets="copy")
+                self.assertTrue(tmp_json_file.exists())
+                another_copy = Task.Task.from_json_file(tmp_json_file, self.assets)
                 # This does not test 100% for equality but should cover the most central sanity checks
-                self.assertEqual(task.header, another_copy.header)
-                self.assertEqual(set(o.id for o in task.obstacles), set(o.id for o in another_copy.obstacles))
-                self.assertEqual(set(g.id for g in task.goals), set(g.id for g in another_copy.goals))
-                self.assertEqual(set(type(c) for c in task.constraints),
-                                 set(type(c) for c in another_copy.constraints))
+                self.assertEqual(task, another_copy)
 
     def test_pickle_robot(self):
         """Tests whether the kinematic model remains when pickling - geometry cannot be preserved at the moment"""
@@ -466,7 +544,7 @@ class SerializationTests(unittest.TestCase):
 
         # Test with more complex modular robot that also has joint inertias / friction / ...
         modular_complex = ModuleAssembly.from_serial_modules(
-            ModulesDB.from_file(*get_module_db_files('IMPROV')),
+            ModulesDB.from_name('IMPROV'),
             ["1", "21", "14", "21", "14", "23", "13"]
         ).robot
         fresh_robot = ModuleAssembly.from_monolithic_robot(modular_complex).robot
@@ -476,15 +554,15 @@ class SerializationTests(unittest.TestCase):
         self.assertTrue(pin_geometry_models_structurally_equal(modular_complex.visual, fresh_robot.visual))
 
     def test_urdf2json(self):
-        assembly = ModuleAssembly.from_monolithic_robot(self.robot, self.panda_urdf)
+        assembly = ModuleAssembly.from_monolithic_robot(self.robot)
         db_data = assembly.db.to_json_data()
-        reconstructed_db = ModulesDB.from_json_data(db_data, self.panda_assets, "panda")
+        reconstructed_db = ModulesDB.from_json_data(db_data, "panda")
         assembly_from_json = ModuleAssembly.from_serial_modules(reconstructed_db, ("panda",))
-        with tempfile.NamedTemporaryFile() as f:
-            assembly.to_urdf(write_to=Path(f.name))
-            f.flush()
+        with tempfile.TemporaryDirectory() as d:
+            urdf_file = Path(d, "panda.urdf")
+            assembly.to_urdf(write_to=urdf_file, handle_missing_assets="symlink")
             assembly_from_urdf = ModuleAssembly.from_monolithic_robot(
-                Robot.PinRobot.from_urdf(Path(f.name), self.panda_assets))
+                Robot.PinRobot.from_urdf(urdf_file, urdf_file.parent))
 
         for recreated_assembly in (assembly_from_json, assembly_from_urdf):
             self.assertTrue(pin_models_functionally_equal(self.robot.model, recreated_assembly.robot.model))
